# Manage a Weblate service
class tails::weblate (
  Array[String] $admins,
  String $db_password,
  String $weblate_secret_key,
  String $redis_password,
  String $db_user                         = 'weblate',
  String $db_name                         = 'weblate',
  String $code_git_remote                 = 'https://github.com/WeblateOrg/weblate.git',
  String $code_git_revision               = 'weblate-3.5.1',
  Stdlib::Absolutepath $apache_data_dir   = '/var/www/weblate',
  Stdlib::Absolutepath $tmserver_data_dir = '/var/lib/tmserver',
  Hash $po_slave_languages                = $tails::website::params::weblate_slave_languages,
) inherits tails::website::params {

  ### Constants
  #
  # Their value is hard-coded in scripts that are hard to maintain
  # as templates, which is why they are not class parameters.
  # Passing these values as arguments to the aforementioned scripts
  # would be doable but require considerable effort for little value.

  $code_git_checkout = '/usr/local/share/weblate'
  $logs_dir          = '/var/log/weblate'
  $mutable_data_dir  = '/var/lib/weblate'

  ### Sanity checks

  if $::lsbdistcodename != 'stretch' {
    fail('The tails::weblate class only supports Debian Stretch.')
  }

  ### Resources

  include tails::website::builder

  # Dependencies

  include tails::weblate::dependencies

  # Users and groups

  user { 'tmserver':
    ensure => present,
    system => true,
    home   => $tmserver_data_dir,
    gid    => 'tmserver',
  }

  user { 'weblate':
    ensure => present,
    system => true,
    home   => $mutable_data_dir,
    gid    => 'weblate',
  }

  group { 'tmserver':
    ensure => present,
    system => true,
  }

  group { 'weblate':
    ensure => present,
    system => true,
  }

  group { 'weblate_admin':
    ensure => present,
    system => true,
  }

  tails::weblate::admin { $admins: }

  # sudo credentials

  sudo::conf { 'weblate-admin':
    source  => 'puppet:///modules/tails/weblate/sudo/weblate-admin',
    require => Group['weblate_admin'],
  }

  sudo::conf { 'puppeteers':
    source  => 'puppet:///modules/tails/weblate/sudo/puppeteers',
    require => User['hefee'],
  }

  # static data served by Apache without going through wsgi

  file { $apache_data_dir:
    ensure => directory,
    owner  => root,
    group  => weblate,
    mode   => '2755',
  }

  file { "${apache_data_dir}/media":
    ensure => directory,
    owner  => root,
    group  => weblate,
    mode   => '2775',
  }

  file { "${apache_data_dir}/static":
    ensure => directory,
    owner  => root,
    group  => weblate,
    mode   => '2775',
  }

  # Mutable data

  file { $mutable_data_dir:
    ensure => directory,
    owner  => root,
    group  => weblate,
    mode   => '2750',
  }

  file { "${mutable_data_dir}/repositories":
    ensure => directory,
    owner  => root,
    group  => weblate,
    mode   => '2770',
  }

  # Repository used by weblate
  vcsrepo { "${mutable_data_dir}/repositories/vcs/tails/index":
    ensure   => present,
    provider => git,
    remote   => 'origin',
    user     => weblate,
    group    => weblate,
    source   => {
      # used by Weblate
      'origin' => 'gitolite@puppet-git.lizard:tails',
      # update_weblate_components.py, that operates on this working copy,
      # needs to keep track of the status of the remote repo independently
      # of Weblate
      'cron'   => 'gitolite@puppet-git.lizard:tails',
    },
  }

  # Repository used by our scripts that update Git outside of Weblate
  vcsrepo { "${mutable_data_dir}/repositories/integration":
    ensure   => present,
    provider => git,
    remote   => 'origin',
    user     => weblate,
    group    => weblate,
    source   => {
      # used to merge/push changes from/to the canonical repo
      'origin'  => 'gitolite@puppet-git.lizard:tails',
      # used to fetch the commits that Weblate created
      'weblate' => "${mutable_data_dir}/repositories/vcs/tails/index/.git",
    },
    require  => Vcsrepo["${mutable_data_dir}/repositories/vcs/tails/index"],
  }

  $git_checkouts = [
    "${mutable_data_dir}/repositories/vcs/tails/index",
    "${mutable_data_dir}/repositories/integration",
  ]
  $git_checkouts.each |String $git_checkout| {
    $git_config = "${git_checkout}/.git/config"
    file { $git_config:
      owner   => weblate,
      group   => weblate,
      mode    => '0640',
      require => Vcsrepo[$git_checkout],
    }
    create_ini_settings(
      {
        'user' => {
          'name'  => 'Tails translators',
          'email' => 'tails-l10n@boum.org',
        },
      },
      {
        path    => $git_config,
        require => File[$git_config],
      }
    )
  }

  file { "${mutable_data_dir}/.ssh":
    ensure => directory,
    owner  => weblate,
    group  => weblate,
    mode   => '2770',
  }

  # Machine Translation Service

  file { $tmserver_data_dir:
    ensure => directory,
    owner  => tmserver,
    group  => 'weblate',
    mode   => '2771',
  }

  # Code

  vcsrepo { $code_git_checkout:
    provider => git,
    source   => $code_git_remote,
    revision => $code_git_revision,
    user     => root,
  }

  # Service

  service { 'apache2':
    ensure  => running,
    require => Package[apache2],
  }

  service { 'memcached':
    ensure  => running,
    require => Package[memcached],
  }


  # Configuration

  file { "${mutable_data_dir}/config":
    ensure => directory,
    owner  => root,
    group  => 'weblate_admin',
    mode   => '2775',
  }

  file { "${mutable_data_dir}/config/apache-vhost.conf":
    ensure  => present,
    content => template('tails/weblate/apache-vhost.erb'),
    owner   => root,
    group   => 'weblate_admin',
    mode    => '0664',
    notify  => Service[apache2],
  }
  file { '/etc/apache2/sites-available/000-default.conf':
    ensure => symlink,
    target => "${mutable_data_dir}/config/apache-vhost.conf",
    notify => Service[apache2],
  }
  file { '/etc/modsecurity/modsecurity.conf':
    source => 'puppet:///modules/tails/weblate/modsecurity/modsecurity.conf',
    ensure => present,
    owner  => root,
    group  => root,
    mode   => '0644',
    notify => Service[apache2],
  }
  file { '/etc/modsecurity/crs/crs-setup.conf':
    source => 'puppet:///modules/tails/weblate/modsecurity/crs-setup.conf',
    ensure => present,
    owner  => root,
    group  => root,
    mode   => '0644',
    notify => Service[apache2],
  }

  file { "${mutable_data_dir}/config/settings.py":
    ensure  => present,
    content => template('tails/weblate/settings.py.erb'),
    owner   => 'weblate',
    group   => 'weblate_admin',
    mode    => '0640',  # contains passwords
    notify  => Service[apache2],
  }

  file { "${code_git_checkout}/weblate/settings.py":
    ensure  => symlink,
    target  => "${mutable_data_dir}/config/settings.py",
    require => Vcsrepo[$code_git_checkout],
    notify  => Service[apache2],
  }

  file { "${mutable_data_dir}/scripts":
    ensure => directory,
    owner  => root,
    group  => weblate,
    mode   => '2770',
  }

  # Logs

  file { $logs_dir:
    ensure => directory,
    owner  => root,
    group  => weblate,
    mode   => '2770',
  }

  file { "${mutable_data_dir}/logs":
    ensure => link,
    target => $logs_dir,
  }

  file { '/etc/logrotate.d/weblate':
    source  => 'puppet:///modules/tails/weblate/logrotate/logrotate.conf',
    owner   => root,
    group   => root,
    mode    => '0644',
    require => File[$logs_dir],
  }

  # script for upgrading the core pages component list in Weblate
  file { "${mutable_data_dir}/scripts/update_core_pages.py":
    source  => 'puppet:///modules/tails/weblate/scripts/update_core_pages.py',
    mode    => '0750',
    owner   => weblate,
    group   => weblate_admin,
    require => File[
      "${mutable_data_dir}/config/updateCorePages.conf",
      "${mutable_data_dir}/scripts/tailsWeblate.py"
    ],
  }
  file { "${mutable_data_dir}/config/updateCorePages.conf":
    source => 'puppet:///modules/tails/weblate/config/updateCorePages.conf',
    mode   => '0640',
    owner  => weblate,
    group  => weblate_admin,
  }

  # create temporary git repository with all suggestions for staging wiki
  file { "${mutable_data_dir}/scripts/save-suggestions.py":
    source  => 'puppet:///modules/tails/weblate/scripts/save-suggestions.py',
    mode    => '0750',
    owner   => weblate,
    group   => weblate_admin,
    require => File["${mutable_data_dir}/config/saveSuggestions.conf"],
  }
  file { "${mutable_data_dir}/config/saveSuggestions.conf":
    source => 'puppet:///modules/tails/weblate/config/saveSuggestions.conf',
    mode   => '0640',
    owner  => weblate,
    group  => weblate_admin,
  }

  # script for upgrading git repository in weblate
  file { "${mutable_data_dir}/scripts/update_weblate_components.py":
    source  => 'puppet:///modules/tails/weblate/scripts/update_weblate_components.py',
    mode    => '0750',
    owner   => weblate,
    group   => weblate_admin,
    require => File[
      "${mutable_data_dir}/config/updateWeblateComponents.conf",
      "${mutable_data_dir}/scripts/tailsWeblate.py",
      "${mutable_data_dir}/scripts/merge_canonical_changes.py"
    ],
  }
  file { "${mutable_data_dir}/config/updateWeblateComponents.conf":
    source => 'puppet:///modules/tails/weblate/config/updateWeblateComponents.conf',
    mode   => '0640',
    owner  => weblate,
    group  => weblate_admin,
  }

  # script to update the weblate git repository from main git
  file { "${mutable_data_dir}/scripts/merge_canonical_changes.py":
    source  => 'puppet:///modules/tails/weblate/scripts/merge_canonical_changes.py',
    mode    => '0750',
    owner   => weblate,
    group   => weblate_admin,
    require => File["${mutable_data_dir}/config/mergeCanonicalChanges.conf"],
  }
  file { "${mutable_data_dir}/config/mergeCanonicalChanges.conf":
    source => 'puppet:///modules/tails/weblate/config/mergeCanonicalChanges.conf',
    mode   => '0640',
    owner  => weblate,
    group  => weblate_admin,
  }

  file { "${mutable_data_dir}/config/langs.json":
    content => template('tails/weblate/langs.json.erb'),
    mode    => '0640',
    owner   => weblate,
    group   => weblate_admin,
  }

  # script to update the weblate git repository from main git
  file { "${mutable_data_dir}/scripts/merge_weblate_changes.py":
    source  => 'puppet:///modules/tails/weblate/scripts/merge_weblate_changes.py',
    mode    => '0750',
    owner   => weblate,
    group   => weblate_admin,
    require => File[
      "${mutable_data_dir}/config/mergeWeblateChanges.conf",
      "${mutable_data_dir}/scripts/merge_canonical_changes.py"
    ],
  }
  file { "${mutable_data_dir}/config/mergeWeblateChanges.conf":
    source => 'puppet:///modules/tails/weblate/config/mergeWeblateChanges.conf',
    mode   => '0640',
    owner  => weblate,
    group  => weblate_admin,
  }


  # script to check/update Weblate components against git.
  file { "${mutable_data_dir}/scripts/weblate_status.py":
    source => 'puppet:///modules/tails/weblate/scripts/weblate_status.py',
    mode   => '0750',
    owner  => weblate,
    group  => weblate_admin,
  }

  # commonly used functions to handle Weblate components.
  file { "${mutable_data_dir}/scripts/tailsWeblate.py":
    source => 'puppet:///modules/tails/weblate/scripts/tailsWeblate.py',
    mode   => '0640',
    owner  => weblate,
    group  => weblate_admin,
  }

  # script to check Weblate role permissions.
  file { "${mutable_data_dir}/scripts/check_weblate_permission.py":
    source => 'puppet:///modules/tails/weblate/scripts/check_weblate_permission.py',
    mode   => '0750',
    owner  => weblate,
    group  => weblate_admin,
    require => File[
      "${mutable_data_dir}/config/weblate_permission.yaml",
      "${mutable_data_dir}/scripts/tailsWeblate.py"
    ],
  }
  file { "${mutable_data_dir}/config/weblate_permission.yaml":
    source => 'puppet:///modules/tails/weblate/config/weblate_permission.yaml',
    mode   => '0640',
    owner  => weblate,
    group  => weblate_admin,
  }



  # ikiwiki setup for staging wiki
  $src_dir = "${mutable_data_dir}/repositories/vcs/staging/wiki/src"
  $git_dir = ''
  $web_dir = '/var/www/staging'
  $url = 'https://staging.tails.boum.org'
  $cgi_url = "${url}/ikiwiki.cgi"
  $is_staging = true
  $underlays = []

  file { "${mutable_data_dir}/config/ikiwiki.setup":
    content => template('tails/website/ikiwiki.setup.erb'),
    mode    => '0644',
    owner   => weblate,
    group   => weblate_admin,
  }

  # Cronjobs

  file { "${mutable_data_dir}/scripts/update_tm.sh":
    content => template('tails/weblate/update_tm.sh.erb'),
    mode    => '0750',
    owner   => weblate,
    group   => weblate_admin,
  }

  cron { 'weblate update tmserver':
    command  => "${mutable_data_dir}/scripts/update_tm.sh",
    user     => weblate,
    monthday => 1,
    hour     => 5,
    minute   => 23,
    require  => File["${mutable_data_dir}/scripts/update_tm.sh"],
  }

  file { "${mutable_data_dir}/scripts/update-staging-website.sh":
    source => 'puppet:///modules/tails/weblate/scripts/update-staging-website.sh',
    mode   => '0750',
    owner  => weblate,
    group  => weblate_admin,
  }

  cron { 'weblate update staging':
    command => "${mutable_data_dir}/scripts/update-staging-website.sh",
    user    => weblate,
    hour    => 3,
    minute  => 21,
    require => File["${mutable_data_dir}/scripts/update-staging-website.sh"],
  }

  file { "${mutable_data_dir}/scripts/cron.sh":
    source  => 'puppet:///modules/tails/weblate/scripts/cron.sh',
    mode    => '0750',
    owner   => weblate,
    group   => weblate_admin,
    require => [
      Vcsrepo[
        "${mutable_data_dir}/repositories/vcs/tails/index",
        "${mutable_data_dir}/repositories/integration",
        $code_git_checkout,
      ],
      File[
        $logs_dir,
        "${mutable_data_dir}/scripts/merge_canonical_changes.py",
        "${mutable_data_dir}/scripts/update_weblate_components.py",
        "${mutable_data_dir}/scripts/merge_weblate_changes.py",
      ],
    ],
  }

  cron { 'weblate cronjobs':
    command => "cd ${mutable_data_dir}/scripts && ./cron.sh",
    user    => weblate,
    minute  => '*/5',
    require => File["${mutable_data_dir}/scripts/cron.sh"],
  }

  class { 'tails::weblate::celery':
    logs_dir          => $logs_dir,
    code_git_checkout => $code_git_checkout,
    redis_password    => $redis_password,
  }

  # Database

  class { '::mysql::server':
    override_options => {
      'mysqld' => {
        'sql_mode'                     => 'NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION,STRICT_ALL_TABLES',
        'query_cache_type'             => '1',
        'query_cache_size'             => '32M',
        'tmp_table_size'               => '32M',
        'max_heap_table_size'          => '32M',
        'innodb_buffer_pool_size'      => '2G',
        'innodb_buffer_pool_instances' => '2',
      },
    },
  }

  mysql::db { $db_name:
    charset  => 'utf8mb4',
    collate  => 'utf8mb4_unicode_ci',
    user     => $db_user,
    password => $db_password,
    host     => 'localhost',
    grant    => ['ALL'],
  }

  # TM Server

  file { '/usr/local/bin/tmserver':
    ensure  => file,
    mode    => '0755',
    owner   => root,
    group   => staff,
    require => Tails::Pip_package_from_repo['translate-toolkit'],
  }

  file { "${tmserver_data_dir}/db":
    ensure => present,
    owner  => 'weblate',
    group  => 'tmserver',
    mode   => '0664',
  }

  file { '/etc/systemd/system/tmserver.service':
    owner   => root,
    group   => root,
    mode    => '0644',
    require => File[
      '/usr/local/bin/tmserver',
      "${tmserver_data_dir}/db",
    ],
    content => "[Unit]
Description=TM Server Instance

[Service]
User=tmserver
Group=tmserver
ExecStart=/usr/local/bin/tmserver -d ${tmserver_data_dir}/db -b 127.0.0.1 -p 8080

[Install]
WantedBy=multi-user.target
",
  }

  service { 'tmserver':
    ensure    => running,
    provider  => systemd,
    require   => [
      File['/etc/systemd/system/tmserver.service']
    ],
    subscribe => File['/etc/systemd/system/tmserver.service'],
  }
  exec { 'systemctl enable tmserver.service':
    creates => '/etc/systemd/system/multi-user.target.wants/tmserver.service',
  }

  # Backups

  include tails::backupninja
  file { '/etc/backup.d/10.mysql':
    ensure  => present,
    owner   => root,
    group   => root,
    mode    => '0400',
    content => "hotcopy = no
sqldump = yes
backupdir = /var/backups/mysql
databases = all
configfile = /etc/mysql/debian.cnf
",
    require => Package['backupninja'],
  }

  file { '/etc/backup.d/20.sh':
    ensure  => present,
    owner   => root,
    group   => root,
    mode    => '0400',
    content => "/usr/bin/sqlite3 ${tmserver_data_dir}/db \".backup '/var/backups/tmserver.db'\"",
    require => Package['sqlite3'],
  }
}
